<?php

namespace ConsentMagicPro;

if ( !defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

use MaxMind\Db\Reader;
use MaxMind\Db\Reader\InvalidDatabaseException;

class CS_Geoip {

	public $version;

	public $download_url;

	public $databases = array();

	public $database_extension;

	public function __construct() {

		include_once( ABSPATH . 'wp-admin/includes/plugin.php' );

		$this->databases          = array(
			'city'    => 'GeoLite2-City',
			'country' => 'GeoLite2-Country'
		);
		$this->download_url       = 'https://download.maxmind.com/app/geoip_download';
		$this->database_extension = '.mmdb';

	}

	public function cs_check_country() {

		$ipAddress           = get_client_ip();
		$databaseCountryFile = CMPRO_PLUGIN_PATH . 'includes/modules/geoip2/GeoLite2-Country.mmdb';
		$Country             = false;

		if ( file_exists( $databaseCountryFile ) && !empty( $ipAddress ) ) {

			try {
				$reader       = new Reader( $databaseCountryFile );
				$Country_data = $reader->get( $ipAddress );
				if ( isset( $Country_data[ 'country' ][ 'iso_code' ] ) ) {
					$Country = $Country_data[ 'country' ][ 'iso_code' ];
				} elseif ( isset( $Country_data[ 'registered_country' ][ 'iso_code' ] ) ) {
					$Country = $Country_data[ 'registered_country' ][ 'iso_code' ];
				}
				$reader->close();

			} catch ( InvalidDatabaseException $e ) {
				error_log( $e->getMessage() );
				$Country = false;
				db_cron_update_one_time();
			} catch ( \BadMethodCallException|\Exception $e ) {
				error_log( $e->getMessage() );
				$Country = false;
			}
		}

		return $Country;
	}

	public function cs_check_city() {

		$ipAddress        = get_client_ip();
		$databaseCityFile = CMPRO_PLUGIN_PATH . 'includes/modules/geoip2/GeoLite2-City.mmdb';
		$City             = false;
		if ( file_exists( $databaseCityFile ) && !empty( $ipAddress ) ) {

			try {
				$reader    = new Reader( $databaseCityFile );
				$City_data = $reader->get( $ipAddress );
				$City      = $City_data[ 'subdivisions' ][ 0 ][ 'names' ][ 'en' ] ?? false;
				$reader->close();

			} catch ( InvalidDatabaseException $e ) {
				error_log( $e->getMessage() );
				$City = false;
				db_cron_update_one_time();
			} catch ( \BadMethodCallException|\Exception $e ) {
				error_log( $e->getMessage() );
				$City = false;
			}
		}

		return $City;
	}

	public function cs_validate_geo_license_key( $key ) {

		$key = trim( stripslashes( $key ) );

		// if empty license keys return WP_Error.
		if ( empty( $key ) ) {
			return new \WP_Error( 'geo_error', esc_html__( 'License key is empty.', 'consent-magic' ) );
		}

		// Check the license key by attempting to download the Geolocation database.
		$tmp_database_path = $this->cs_download_database( $key, $this->databases[ 'country' ] );
		if ( is_wp_error( $tmp_database_path ) ) {
			return $tmp_database_path;
		}

		// Downloading DB
		$tmp_database_path = $this->update_database( $this->databases[ 'country' ], $tmp_database_path );

		return $tmp_database_path;
	}

	//download maxmindDB when settings saved
	public function cs_download_database( $key, $database ) {

		if ( is_plugin_activated() ) {

			$download_uri = add_query_arg( array(
				'edition_id'  => $database,
				'license_key' => urlencode( sanitize_text_field( $key ) ),
				'suffix'      => 'tar.gz',
			), $this->download_url );

			// Needed for the download_url call right below.
			require_once ABSPATH . 'wp-admin/includes/file.php';

			$tmp_archive_path = download_url( esc_url_raw( $download_uri ) );
			if ( is_wp_error( $tmp_archive_path ) ) {
				// Transform the error into something more informative.
				$error_data = $tmp_archive_path->get_error_data();
				if ( isset( $error_data[ 'code' ] ) ) {
					switch ( $error_data[ 'code' ] ) {
						case 401:
							return new \WP_Error( 'geo_error', esc_html__( 'The MaxMind license key is invalid.', 'consent-magic' ) );
					}
				}

				return new \WP_Error( 'geo_error', esc_html__( 'Failed to download the MaxMind database.', 'consent-magic' ) );
			}

			// Extract the database from the archive.
			$tmp_database_path = '';
			try {
				$file = new \PharData( $tmp_archive_path );

				$tmp_database_path = trailingslashit( dirname( $tmp_archive_path ) ) . trailingslashit( $file->current()->getFilename() ) . $database . $this->database_extension;

				$file->extractTo( dirname( $tmp_archive_path ), trailingslashit( $file->current()->getFilename() ) . $database . $this->database_extension, true );
			} catch ( \Exception $exception ) {
				error_log( $exception->getMessage() );
			} finally {
				// Remove the archive since we only care about a single file in it.
				unlink( $tmp_archive_path );
			}
		} else {
			return new \WP_Error( 'geo_error', esc_html__( 'Consent Magic license is invalid.', 'consent-magic' ) );
		}

		return $tmp_database_path;
	}

	public function update_database( $database, $new_database_path = null ) {
		// Allow us to easily interact with the filesystem.
		require_once ABSPATH . 'wp-admin/includes/file.php';
		if ( !WP_Filesystem() ) {
			WP_Filesystem();
		}
		global $wp_filesystem;

		$target_database_path = trailingslashit( CMPRO_PLUGIN_PATH ) . trailingslashit( ConsentMagic()->getOption( 'cs_geo_db_file_path' ) ) . $database . $this->database_extension;

		// If there's no database path, we can't store the database.
		if ( empty( $target_database_path ) ) {
			return new \WP_Error( 'geo_error', esc_html__( 'Database path is empty.', 'consent-magic' ) );
		}

		if ( $wp_filesystem->exists( $target_database_path ) ) {
			$wp_filesystem->delete( $target_database_path );
		}

		if ( isset( $new_database_path ) ) {
			$tmp_database_path = $new_database_path;
		} else {
			// We can't download a database if there's no license key configured.
			$license_key = ConsentMagic()->getOption( 'cs_geo_licence_key' );
			if ( empty( $license_key ) ) {
				return new \WP_Error( 'geo_error', esc_html__( 'License key is empty.', 'consent-magic' ) );
			}

			$tmp_database_path = $this->cs_download_database( $license_key, $database );
			if ( is_wp_error( $tmp_database_path ) ) {
				return $tmp_database_path;
			}
		}

		// Move the new database into position.
		$wp_filesystem->move( $tmp_database_path, $target_database_path, true );
		$wp_filesystem->delete( dirname( $tmp_database_path ) );
	}

}

new CS_Geoip();